# -*- coding: utf-8 -*-

from aiohttp import web

from aio_space_track_api import AsyncSpaceTrackApi


async def query(request):
    params = {mvk: request.query.getall(mvk) for mvk in request.query.keys()}

    result = await app['space_track'].query(**params)
    if isinstance(result, (dict, list)):
        return web.json_response(result)
    elif isinstance(result, str):
        return web.Response(text=result)
    return web.Response(body=result)


async def start_space_track(app):
    app['space_track'] = AsyncSpaceTrackApi(loop=app.loop, login='<YOUR_LOGIN>', password='<YOUR_PASSWORD>')
    await app['space_track'].login()


async def cleanup_space_track(app):
    await app['space_track'].logout()
    app['space_track'].session.close()


if __name__ == '__main__':
    app = web.Application()
    app.on_startup.append(start_space_track)
    app.on_cleanup.append(cleanup_space_track)
    app.router.add_get('/', query)
    web.run_app(app, port=8080)



# -*- coding: utf-8 -*-

import logging

DEFAULT_LOG_FORMAT = '[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d] %(message)s'
DEFAULT_LOG_DATE_FORMAT = '%Y-%m-%d %H:%M:%S'

logger = logging.getLogger('aio_space_track_api')



# -*- coding: utf-8 -*-

import asyncio
from pprint import pprint

from aio_space_track_api import AsyncSpaceTrackApi


async def main(loop):
    async with AsyncSpaceTrackApi(loop=loop, login='<YOUR_LOGIN>', password='<YOUR_PASSWORD>') as api:
        tle_list = await api.tle(EPOCH='>now-3',
                                 NORAD_CAT_ID=(25544, 25541,),
                                 order_by=('EPOCH desc', 'NORAD_CAT_ID',),
                                 predicate=('EPOCH', 'NORAD_CAT_ID', 'TLE_LINE0', 'TLE_LINE1', 'TLE_LINE2',))
        pprint(tle_list, indent=2)


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main(loop))



# -*- coding: utf-8 -*-
import asyncio
import threading
import time
from functools import wraps

from ratelimiter import RateLimiter


class AsyncRateLimiter(RateLimiter):
    def __call__(self, func):
        @wraps(func)
        async def async_inner_wrapper(*args, **kwargs):
            async with self:
                return await func(*args, **kwargs)

        return async_inner_wrapper if asyncio.iscoroutinefunction(func) else super().__call__

    def __enter__(self):
        self._lock.acquire()
        # We want to ensure that no more than max_calls were run in the allowed
        # period. For this, we store the last timestamps of each call and run
        # the rate verification upon each __enter__ call.
        if len(self.calls) >= self.max_calls:
            until = time.time() + self.period - self._timespan
            if self.callback:
                t = threading.Thread(target=self.callback, args=(until,))
                t.daemon = True
                t.start()
            sleeptime = until - time.time()
            if sleeptime > 0:
                time.sleep(sleeptime)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        # Store the last operation timestamp.
        self.calls.append(time.time())

        # Pop the timestamp list front (ie: the older calls) until the sum goes
        # back below the period. This is our 'sliding period' window.
        while self._timespan >= self.period:
            self.calls.popleft()

        self._lock.release()

    async def __aenter__(self):
        await self._alock.acquire()

        # We want to ensure that no more than max_calls were run in the allowed
        # period. For this, we store the last timestamps of each call and run
        # the rate verification upon each __enter__ call.
        if len(self.calls) >= self.max_calls:
            until = time.time() + self.period - self._timespan
            if self.callback:
                asyncio.ensure_future(self.callback(until))
            sleeptime = until - time.time()
            if sleeptime > 0:
                await asyncio.sleep(sleeptime)
        return self

    async def __aexit__(self, *args):
        self.calls.append(time.time())
        # Pop the timestamp list front (ie: the older calls) until the sum goes
        # back below the period. This is our 'sliding period' window.
        while self._timespan >= self.period:
            self.calls.popleft()

        self._alock.release()



# -*- coding: utf-8 -*-

import codecs
import re
import sys
from setuptools import setup

import os

if sys.version_info < (3, 5, 0):
    raise RuntimeError("aio-space-track-api requires Python 3.5.0+")

PROJECT_DIR = os.path.abspath(os.path.dirname(__file__))
VERSION_REGEXP = re.compile(r"^__version__ = [\'\"](.+?)[\'\"]$", re.MULTILINE)


def read(fn):
    with codecs.open(os.path.join(PROJECT_DIR, fn), encoding='utf-8') as f:
        return f.read().strip()


def version():
    try:
        return VERSION_REGEXP.findall(read(os.path.join('aio_space_track_api', '__init__.py')))[0]
    except IndexError:
        raise RuntimeError('Unable to determine version.')


vn = version()
url = 'https://github.com/nkoshell/aio-space-track-api'

setup(
    name='aio-space-track-api',
    description='Small async wrapper for "space-track-api" package.',
    long_description=read('README.rst'),
    version=vn,
    packages=['aio_space_track_api'],
    url=url,
    download_url='{url}/archive/{version}.tar.gz'.format(url=url, version=vn),
    license='MIT',
    author='NikitaKoshelev',
    author_email='nikita.koshelev@gmail.com',
    install_requires=['aiohttp>=2.0.7', 'space-track-api>=1.0.2'],
)



# -*- coding: utf-8 -*-
import logging

from aiohttp import ClientSession, ClientResponseError
from space_track_api import SpaceTrackApi, SpaceTrackQueryBuilder

from .utils import AsyncRateLimiter


class AsyncSpaceTrackApi(SpaceTrackApi):
    def __init__(self, login, password, session=None, loop=None, **kwargs):
        kwargs.setdefault('logger', logging.getLogger('{}.{}'.format(__name__, self.__class__.__name__)))

        super().__init__(login, password, **kwargs)

        self.session = session if isinstance(session, ClientSession) else ClientSession(loop=loop)

    @AsyncRateLimiter(max_calls=20, period=60)
    async def query(self, **kwargs):
        qb = SpaceTrackQueryBuilder(**kwargs)
        url = '{url}/{query}'.format(url=self.url, query=qb)
        self.logger.info('Send request to %s', url)
        async with self.session.get(url) as resp:
            m = self.get_response_method(qb.format)
            try:
                return await getattr(resp, m)()
            except (AttributeError, ClientResponseError) as e:
                self.logger.exception(e)
                return await resp.text()

    async def login(self):
        async with self.session.post('{}/{}'.format(self.url, self.login_url), data=self.credentials) as resp:
            if resp.reason == 'OK':
                self.logger.info('"Successfully logged in"')
                return self.session

    async def logout(self):
        async with self.session.get('{}/{}'.format(self.url, self.logout_url)) as resp:
            if resp.reason == 'OK':
                self.logger.info(await resp.text())

    @staticmethod
    def get_response_method(fmt):
        method_mapping = {
            'json': 'json',
            'xml': 'text',
            'html': 'text',
            'csv': 'text',
            'tle': 'text',
            '3le': 'text',
            'kvn': 'text',
            'stream': 'read'
        }
        return method_mapping.get(fmt, 'read')

    async def __call__(self, **kwargs):
        return await self.query(**kwargs)

    def __enter__(self):
        raise NotImplementedError('Only async use')

    async def __aenter__(self):
        await self.login()
        return self

    async def __aexit__(self, *args):
        await self.logout()
        self.close()
